package com.abi.qms.platform.controller.console;

import java.util.List;
import java.util.Arrays;

import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.GetInspectionCaseInfoRes;
import com.abi.qms.platform.dto.res.PageListInspectionCaseRes;
import com.abi.qms.platform.service.IInspectionCaseService;
import com.abi.task.common.api.base.BaseResponse;
import lombok.RequiredArgsConstructor;

import javax.validation.Valid;
import javax.validation.constraints.*;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 稽查案件 Controller
 *
 * @author: ZhangZhao
 * @date: 2021-07-19
 */
@Slf4j
@Api(value = "稽查案件 控制器", tags = {"稽查案件 管理"})
@RestController
@RequestMapping("/inspectionCase")
public class InspectionCaseController {

    @Autowired
    private IInspectionCaseService inspectionCaseService;


    /**
     * 新增稽查案件
     */
    @ApiOperation("新增稽查案件 ")
    @PostMapping("/save")
    public BaseResponse addInspectionCase(@Validated @RequestBody AddInspectionCaseReq addReq) {
        inspectionCaseService.addInspectionCase(addReq);
        return BaseResponse.create();
    }


    /**
     * 查询稽查案件 列表
     */
    @ApiOperation("查询稽查案件 列表")
    @GetMapping("/page/list")
    public BaseResponse<PageListInspectionCaseRes> pageList(@Validated PageListInspectionCaseReq pageListReq) {
        PageListInspectionCaseRes res = inspectionCaseService.pageListQuery(pageListReq);
        return BaseResponse.create(res);
    }


    /**
     * 获取稽查案件 详细信息
     */
    @ApiOperation("获取稽查案件 详细信息")
    @GetMapping("/details/{id}")
    public BaseResponse<GetInspectionCaseInfoRes> getInspectionCaseInfo(@NotNull(message = "主键不能为空")
                                                                        @PathVariable("id") Long id) {
        GetInspectionCaseInfoRes res = inspectionCaseService.getInspectionCaseInfo(id);
        return BaseResponse.create(res);
    }


    /**
     * 修改稽查案件
     */
    @ApiOperation("编辑稽查案件 ")
    @PutMapping("/edit")
    public BaseResponse editInspectionCase(@Validated @RequestBody EditInspectionCaseReq editReq) {
        inspectionCaseService.editInspectionCase(editReq);
        return BaseResponse.create();
    }

    /**
     * 删除稽查案件
     */
    @ApiOperation("删除稽查案件 ")
    @DeleteMapping("/{ids}")
    public BaseResponse removeInspectionCase(@NotEmpty(message = "主键不能为空")
                                             @PathVariable Long[] ids) {
        inspectionCaseService.removeInspectionCase(ids);
        return BaseResponse.create();
    }
}
