package com.abi.qms.platform.dto.req;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.ToString;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.List;

/**
 * @author: ZhangZhao
 * @date: 2021/7/20 14:48
 * @description: 稽查案件 编辑接口入参
 */
@Data
@NoArgsConstructor
@ApiModel
public class EditInspectionCaseReq implements Serializable {

    @NotNull
    @ApiModelProperty(name = "案件ID", required = true)
    private Long id;

    /**
     * 案件备注
     */
    @Size(max = 150)
    @ApiModelProperty(name = "案件备注")
    private String caseRemark;

    /**
     * 案件涉及箱码列表
     */
    @Size(min = 1, message = "案件涉及箱码不能为空")
    @ApiModelProperty(name = "案件涉及箱码列表", required = true)
    private List<EditInspectionCaseReq.BoxCode> involvedBoxCodes;


    /**
     * 拆分数量 BEAN
     */
    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @ToString
    @ApiModel(value = "EditInspectionCaseReq_BoxCode")
    public static class BoxCode implements Serializable {

        /**
         * 箱码
         */
        @NotEmpty(message = "箱码不能为空")
        @Size(max = 32)
        @ApiModelProperty(value = "箱码", required = true)
        private String boxCode;

        /**
         * 手写标记
         */
        @Size(max = 15)
        @ApiModelProperty(value = "手写标记")
        private String handMark;

    }


}
