package com.abi.qms.platform.service.impl;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.abi.qms.platform.dao.entity.FlowQueryRecord;
import com.abi.qms.platform.dao.enums.FlowQueryTypeEnum;
import com.abi.qms.platform.dao.enums.FlowStatusEnum;
import com.abi.qms.platform.dao.enums.ReqChannelTypeEnum;
import com.abi.qms.platform.dao.mapper.FlowQueryRecordMapper;
import com.abi.qms.platform.dao.vo.result.FlowQueryRecordInfoVo;
import com.abi.qms.platform.dao.vo.result.ListFlowQueryRecordVO;
import com.abi.qms.platform.dto.req.AddFlowQueryRecordReq;
import com.abi.qms.platform.dto.req.PageListFlowQueryRecordReq;
import com.abi.qms.platform.dto.res.GetFlowQueryRecordInfo;
import com.abi.qms.platform.dto.res.PageListFlowQueryRecordRes;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.service.IFlowQueryRecordService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 流向查询记录 Service业务层处理
 *
 * @author: ZhangZhao
 * @date: 2021-07-21
 */
@Service
public class FlowQueryRecordServiceImpl implements IFlowQueryRecordService {

    @Autowired
    private FlowQueryRecordMapper flowQueryRecordMapper;


    /**
     * 分页查询列表
     *
     * @param pageListReq 查询条件
     * @return 查询结果
     */
    @Override
    public PageListFlowQueryRecordRes pageListQuery(PageListFlowQueryRecordReq pageListReq) {

        IPage<ListFlowQueryRecordVO> pageResult = flowQueryRecordMapper.pageListQuery(PageUtil.createPage(pageListReq), pageListReq);

        List<ListFlowQueryRecordVO> flowQueryRecordVOList = pageResult.getRecords();

        //转换类型
        List<PageListFlowQueryRecordRes.FlowQueryRecordBean> flowQueryRecordBeanList = flowQueryRecordVOList.stream()
                .parallel().map(s -> {
                    PageListFlowQueryRecordRes.FlowQueryRecordBean flowQueryRecordBean = new PageListFlowQueryRecordRes.FlowQueryRecordBean();
                    BeanUtil.copyProperties(s, flowQueryRecordBean);

                    //流向状态
                    String flowStatusName = FlowStatusEnum.getName(s.getFlowStatus());
                    flowQueryRecordBean.setFlowStatus(flowStatusName);

                    //查询平台
                    String channelName = ReqChannelTypeEnum.getName(s.getQueryPlatform());
                    flowQueryRecordBean.setQueryPlatform(channelName);

                    return flowQueryRecordBean;
                }).collect(Collectors.toList());

        //封装出参
        PageListFlowQueryRecordRes flowQueryRecordRes = new PageListFlowQueryRecordRes();

        //放入分页信息
        PageUtil.copyPageInfo(flowQueryRecordRes, pageResult);
        flowQueryRecordRes.setFlowQueryRecordList(flowQueryRecordBeanList);

        return flowQueryRecordRes;
    }


    /**
     * 新增流向查询记录
     *
     * @param addReq 新增请求
     */
    @Override
    public void addFlowQueryRecord(AddFlowQueryRecordReq addReq) {
        //校验请求内容
        validatedFlowQueryRecordReq(addReq);

        //值拷贝
        FlowQueryRecord flowQueryRecord = new FlowQueryRecord();
        BeanUtil.copyProperties(addReq, flowQueryRecord);

        //PC查询地址信息处理
        Integer queryPlatform = addReq.getQueryPlatform();
        String queryAddress = addReq.getQueryAddress();
        if (ReqChannelTypeEnum.BACKSTAGE.is(queryPlatform)) {
            //根据ip查询地址
            String ip = addReq.getQueryIp();
            queryAddress = this.getAddressByIP(ip);
        }
        flowQueryRecord.setQueryAddress(queryAddress);

        //保存信息
        flowQueryRecordMapper.insert(flowQueryRecord);

    }

    /**
     * 根据ID获取记录详情
     *
     * @param id 指定ID
     * @return 详情信息
     */
    @Override
    public GetFlowQueryRecordInfo getDetailsById(final Long id) {
        FlowQueryRecordInfoVo flowQueryRecordInfoVo = flowQueryRecordMapper.getDetailsById(id);
        AssertUtil.isNull(flowQueryRecordInfoVo, "未找到指定记录！");
        //转换类型
        GetFlowQueryRecordInfo getFlowQueryRecordInfo = new GetFlowQueryRecordInfo();
        BeanUtil.copyProperties(flowQueryRecordInfoVo, getFlowQueryRecordInfo);
        //编号
        getFlowQueryRecordInfo.setNumber(flowQueryRecordInfoVo.getId());
        //流向状态
        Integer flowStatus = flowQueryRecordInfoVo.getFlowStatus();
        String flowStatusName = FlowStatusEnum.getName(flowStatus);
        getFlowQueryRecordInfo.setFlowStatus(flowStatusName);

        return getFlowQueryRecordInfo;
    }


    /**
     * 根据IP查询地址信息
     *
     * @param ip 请求IP
     * @return ip对应地址信息
     */
    private String getAddressByIP(String ip) {
        //TODO   根据IP查询地址
        return "江苏南京";
    }


    /**
     * 校验-新增流向查询记录请求内容
     *
     * @param addReq 请求内容
     */
    private static void validatedFlowQueryRecordReq(AddFlowQueryRecordReq addReq) {
        //流向状态
        Integer flowStatus = addReq.getFlowStatus();
        AssertUtil.isNull(flowStatus, "请指明 流向状态！");

        //查询平台
        Integer queryPlatform = addReq.getQueryPlatform();
        AssertUtil.isNull(flowStatus, "请指明 查询平台！");

        //查询类型
        Integer queryType = addReq.getQueryType();
        AssertUtil.isNull(queryType, "请指明 查询类型！");

        //箱码
        if (FlowQueryTypeEnum.BOX_CODE.is(queryType)) {
            String boxCode = addReq.getBoxCode();
            AssertUtil.isMeets(boxCode, StrUtil::isNotBlank, "箱码查询时  箱码不能为空！");
        }


        //托盘码
        if (FlowQueryTypeEnum.TRAY_CODE.is(queryType)) {
            String trayCode = addReq.getTrayCode();
            AssertUtil.isMeets(trayCode, StrUtil::isNotBlank, "托盘码查询时  托盘码不能为空！");
        }


        //异常节点
        if (FlowStatusEnum.ABNORMAL.is(flowStatus)) {
            String nodeIn = addReq.getAbnormalNodeIn();
            AssertUtil.isMeets(nodeIn, StrUtil::isNotBlank, "流向状态为异常时  必须指明异常流入节点！");

            String nodeOut = addReq.getAbnormalNodeOut();
            AssertUtil.isMeets(nodeOut, StrUtil::isNotBlank, "流向状态为异常时  必须指明异常流出节点！");
        }

        //查询地址
        if (ReqChannelTypeEnum.WECHAT.is(queryPlatform)) {
            String queryAddress = addReq.getQueryAddress();
            AssertUtil.isMeets(queryAddress, StrUtil::isNotBlank, "微信查询 必须指明请求地址！");
        }


        //查询请求IP
        if (ReqChannelTypeEnum.BACKSTAGE.is(queryPlatform)) {
            String queryIp = addReq.getQueryIp();
            AssertUtil.isMeets(queryIp, StrUtil::isNotBlank, "后台查询 必须指明请求IP！");
        }


    }


}
