package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.QrPackageSapOrder;
import com.abi.qms.platform.dao.mapper.QrPackageSapOrderMapper;
import com.abi.qms.platform.dto.req.BatchDeletePackageSapOrderReq;
import com.abi.qms.platform.dto.req.BatchImportPackageSapOrderReq;
import com.abi.qms.platform.service.QrPackageSapOrderService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * @author: fangxinjian
 * @date: 2021/06/04 17:40
 * @description:
 */
@Service
@Slf4j
public class QrPackageSapOrderServiceImpl implements QrPackageSapOrderService {

    @Autowired
    private QrPackageSapOrderMapper qrPackageSapOrderMapper;

    @Override
    public void batchImportPackageSapOrder(BatchImportPackageSapOrderReq req) {

        List<BatchImportPackageSapOrderReq.BatchImportSapOrder> sapOrderList = req.getBatchImportSapOrderList();
        for (BatchImportPackageSapOrderReq.BatchImportSapOrder order : sapOrderList) {
            try {
                QrPackageSapOrder sapOrder = new QrPackageSapOrder();
                sapOrder.setSapOrderCommitTime(order.getSapOrderCommitTime());
                sapOrder.setSapOrderNo(order.getSapOrderNo());

                saveQrPackageSapOrder(sapOrder);
            } catch (Exception e) {
                log.error("导入Sap订单失败", e);
            }
        }

    }

    @Override
    public void batchUpdatePackageSapOrder(BatchImportPackageSapOrderReq req) {

        List<BatchImportPackageSapOrderReq.BatchImportSapOrder> sapOrderList = req.getBatchImportSapOrderList();
        for (BatchImportPackageSapOrderReq.BatchImportSapOrder order : sapOrderList) {
            try {
                QrPackageSapOrder sapOrder = new QrPackageSapOrder();
                sapOrder.setSapOrderCommitTime(order.getSapOrderCommitTime());
                sapOrder.setSapOrderNo(order.getSapOrderNo());

                updateQrPackageSapOrder(sapOrder);
            } catch (Exception e) {
                log.error("修改Sap订单失败", e);
            }
        }

    }

    @Override
    public void batchDeletePackageSapOrder(BatchDeletePackageSapOrderReq req) {

        List<String> sapOrderNoList = req.getSapOrderNoList();
        for (String sapOrderNo : sapOrderNoList) {
            QrPackageSapOrder qrPackageSapOrderInfo = getQrPackageSapOrderInfo(sapOrderNo);
            if (ObjectUtil.isNull(qrPackageSapOrderInfo)) {
                log.info("Sap订单号:{}不存在",sapOrderNo);
                return;
            }
            qrPackageSapOrderMapper.deleteById(qrPackageSapOrderInfo.getId());
        }

    }

    private void updateQrPackageSapOrder(QrPackageSapOrder sapOrder) {

        //根据sap订单号查询订单信息
        QrPackageSapOrder qrPackageSapOrder = getQrPackageSapOrderInfo(sapOrder.getSapOrderNo());

        if (ObjectUtil.isNull(qrPackageSapOrder)) {
            log.info("Sap订单号:{}不存在", sapOrder.getSapOrderNo());
            return;
        }

        sapOrder.setId(qrPackageSapOrder.getId());
        sapOrder.setUpdateTime(LocalDateTime.now());
        qrPackageSapOrderMapper.updateById(sapOrder);
    }

    private void saveQrPackageSapOrder(QrPackageSapOrder sapOrder) {

        //根据sap订单号查询订单信息
        QrPackageSapOrder qrPackageSapOrder = getQrPackageSapOrderInfo(sapOrder.getSapOrderNo());

        if (ObjectUtil.isNotNull(qrPackageSapOrder)) {
            log.info("Sap订单号:{}已存在", sapOrder.getSapOrderNo());
            return;
        }

        qrPackageSapOrderMapper.insert(sapOrder);
    }

    private QrPackageSapOrder getQrPackageSapOrderInfo(String sapOrderNo) {

        QueryWrapper<QrPackageSapOrder> qw = new QueryWrapper<>();
        qw.eq("sap_order_no", sapOrderNo);
        qw.eq("is_delete", 0);

        //根据sap订单号查询码包订单信息
        QrPackageSapOrder qrPackageSapOrder = qrPackageSapOrderMapper.selectOne(qw);

        return qrPackageSapOrder;

    }
}
